/*
  Main.C

  Micro In-System Programmer
  Uros Platise (C) 1997-1999
*/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include "Terminal.h"
#include "MotIntl.h"
#include "AvrAtmel.h"
#ifndef NO_DAPA
# include "AvrDummy.h"
#endif

/* Globals
*/

int argc;
const char** argv;
char* argv_ok;
unsigned verbose_level;

PDevice device;
TMotIntl motintl;
TTerminal terminal;

const char* version = "uisp version 1.0b, Uros Platise (c) 1997-2000";
const char* help_screen =
"Syntax: uisp [-v{=level}] [-h] [--help] [--version] [--hash=perbytes]\n"
"             [-datmel]"
#ifndef NO_DAPA
" [-dapa|asa|stk200|abb|avrisp] [-dlpt=1|2|3|address]\n"
"             [-dcom=1|2|3|4|address] [-dno_poll] [-dvoltage] \n"
"             [-dt_wd_{flash|eeprom}=time]"
#endif
" [-dserial=device]\n"
"             [-dpart=name|no] [-dspeed=1200|2400|4800|9600|19200|38400|57600]"
"\n"
"             [--upload] [--verify] [--erase] [--lock] [if=input_file]\n"
"             [--segment=segname] [--terminal] [-favr_endianbug]\n\n"
"Programming Methods:\n"
"  -datmel      Standard Atmel Serial Programmer/Atmel Low Cost Programmer\n"
#ifndef NO_DAPA
"  -dapa        Direct AVR Parallel Access\n"
"  -dasa        Direct AVR Serial Access\n"
"  -dstk200     Parallel Starter-Kit 200/300\n"
"  -dabb        Altera ByteBlasterMV Parallel Port Download Cable\n"
"  -davrisp     Atmel AVR ISP\n"
"\n"
"Direct Parallel and Serial Device Settings:\n"
"  -dlpt        Set parallel port 1 (0x278), 2 (0x378, default) or 3 (0x3bc)\n"
"  -dcom        Set direct serial port 1 (0x3f8), 2 (0x2f8), 3 (0x3e8)\n"
"               or 4 (0x2e8) (required for -dasa only)\n"
"  -dlongsck    Set long SCK-high cycle (in micro-seconds) for DTR to charge\n"
"               capacitor (-dasa only)\n"
"  -dno-poll    Program without data polling (little slower)\n"
"  -dvoltage    Set timing specs according to the power supply voltage in [V]\n"
"               (default 5.0 V)\n"
"  -dt_wd_flash Set FLASH maximum write delay time in micro-seconds\n"
"  -dt_wd_eeprom Set EEPROM maximum write delay time in micro-seconds\n"
"               Use -v=3 option to see current settings.\n"
#endif
"\n"
"Amtel Low Cost Programmer Serial Device Settings:\n"
"  -dserial     Set serial interface as /dev/cua* (default /dev/avr)\n"
"  -dpart       Set target abbreviated name or number\n"
"               If -dpart is not given programmer's supported devices\n"
"               are listed. Set -dpart=auto for auto-select.\n"
"  -dspeed      Set speed of the serial interface (default 19200)\n"
"\n"
"Functions:\n"
"  --upload     Upload \"intput_file\" to the AVR memory.\n"
"  --verify     Verify \"intput_file\" (processed after the --upload opt.)\n"
"  --erase      Erase device.\n"
"  --lock       Write lock bits.\n"
"  --segment    Set active segment (auto-select for AVA Motorola output)\n"
"  --reset      Reset device only.\n"
"\n"
"Files:\n"
"  if           Input file for the --upload and --verify functions in\n"
"               Motorola S-records (S1 or S2) or 16 bit Intel format\n"
"\n"
"Other Options:\n"
"  -v           Set verbose level (-v equals -v=2, min/max: 0/3, default 1)\n"
"  --hash       Print hash (default is 32 bytes)\n"
"  --help -h    Help\n"
"  --version    Print version information\n"
"  --terminal   Invoke shell-like terminal\n\n"
"  -favr_endianbug      Swap bytes in the flash memory (ava backward comp.)\n"
"\n"
"Report bugs to: uros.platise@ijs.si (http://medo.fov.uni-mb.si/mapp)";


/* Find command line parameter's value.
   It searches the command line parameters of the form:
   
	argv_name=value
	
   Returns pointer to the value. 
*/
const char* GetCmdParam(const char* argv_name, bool value_required=true){
  int argv_name_len = strlen(argv_name);
  for (int i=1; i<argc; i++){
    if (strncmp(argv_name, argv[i], argv_name_len)==0){
      if (argv[i][argv_name_len]==0){
        if (value_required){
	  throw Error_Device("Incomplete parameter", argv[i]);
	}
	argv_ok[i]=1;
        return &argv[i][argv_name_len];	
      }
      if (argv[i][argv_name_len]=='='){
        argv_ok[i]=1;
        return &argv[i][argv_name_len+1];
      }
    }
  }
  return NULL;
}

/* Print Status Information to the Standard Error Output.
*/
bool Info(unsigned _verbose_level, const char* fmt, ...){
  if (_verbose_level > verbose_level){return false;}
  va_list ap;
  va_start(ap,fmt); 
  vfprintf(stderr,fmt,ap);
  va_end(ap);
  return true;
}

/* Print Help to Standard Output.
*/
bool InfoOut(unsigned _verbose_level, const char* fmt, ...){
  if (_verbose_level > verbose_level){return false;}
  va_list ap;
  va_start(ap,fmt); 
  vfprintf(stdout,fmt,ap);
  va_end(ap);
  return true;
}

int main(int _argc, const char* _argv[]){
  int return_val=0;
  argc = _argc;
  argv = _argv;
  verbose_level=1;  
  
  if (argc==1){
    Info(0, "%s: No command specified.\n", argv[0]); exit(1);
  }  
  argv_ok = (char *)malloc(argc);
  for (int i=1; i<argc; i++){argv_ok[i]=0;}    
  
  /* Help Screen? */
  if (GetCmdParam("-h", false) || GetCmdParam("--help", false)){
    InfoOut(0, "%s\n%s\n", version, help_screen);
    return 0;
  }
  if (GetCmdParam("--version", false)){
    Info(0, "%s\n", version);
    return 0;
  }
  
  /* Setup Verbose Level */
  const char *p = GetCmdParam("-v",false);
  if (p!=NULL){
    if (*p==0){verbose_level=2;} else{verbose_level = atoi(p);}
  }

  /* Invoke Terminal or Command Line Batch Processing */
  try{
    const char* val;
#ifndef NO_DAPA  
    if (GetCmdParam("-dapa", false) || 
        GetCmdParam("-dasa", false) ||
        GetCmdParam("-dstk200", false) ||
	GetCmdParam("-dabb", false) ||
	GetCmdParam("-davrisp", false)){
      device = new TAvrDummy();
    } else 
#endif    
    if (GetCmdParam("-datmel", false)){device = new TAvrAtmel();}
    
    /* Check Device's bad command line params. */
    for (int i=1; i<argc; i++){
      if (argv_ok[i]==0 && strncmp(argv[i], "-d", 2)==0){
        Info(0,"Invalid parameter: %s\n", argv[i]); exit(1);
      }
    }    
    if (device()==NULL){
      throw Error_Device("Programming method is not selected.");
    }

    /* Set Current Active Segment */
    if ((val=GetCmdParam("--segment"))!=NULL){
      if (!device->SetSegment(val)){
	Info(0, "--segment=%s: bad segment name\n", val);
      }
    }

    	/* Device Operations: */
    
    if (GetCmdParam("--erase", false)){device->ChipErase();}

    /* Input file */
    if ((val=GetCmdParam("if"))){
      if (GetCmdParam("--upload", false)){motintl.Read(val, true, false);}
      if (GetCmdParam("--verify", false)){motintl.Read(val, false, true);}
    }

    if (GetCmdParam("--reset", false)){device->ChipReset();}
    if (GetCmdParam("--lock", false)){device->WriteLockBits();}

    	/* enter terminal */ 
	
    if (GetCmdParam("--terminal", false)){terminal.Run();}
    
    /* Check bad command line parameters */
    for (int i=1; i<argc; i++){
      if (argv_ok[i]==0){Info(0,"Invalid parameter: %s\n", argv[i]);}
    }  
  } 
  catch(Error_C){perror("Error"); return_val=-1;}
  catch(Error_Device& errDev){errDev.print(); return_val=-2;}
  catch(Error_MemoryRange& x){
    Info(0, "Address out of memory range.\n"); return_val=-3;
  }    
  free(argv_ok);
  return return_val;
}

